using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class MapManager : MonoBehaviour
{
    public List<GameObject> _mapPiecePrefabs;
    float _spawnX = 0;
    float _blockSize = 10;
    public GameObject _player;
    public GameObject _mapPiece;
    public GameObject _mapPieceSmall;
    Queue<GameObject> _mapPiecesAdded;

    public GameObject _treeWithRootPrefab;
    public GameObject _treeWithRootPrefab2;
    public GameObject _rootFromholePrefab;
    float _distanceFromLastTreeMin = 20;
    float _distanceFromLastTreeMax = 50;
    float _maxPiecesToExistAtOnce = 11;

    float renderAheadDistance = 90;

    float _currentTreeX = 0;
    int _totalMapPiecesSpawned = 0;

    Vector2 _startingPointOnRoot = Vector2.zero;
    static MapManager _this;
    public GameObject _enemyPrefab;
    public GameObject _gemPrefab;

    private void Awake()
    {
        _this = this;
        _mapPiecePrefabs = new List<GameObject>();
        _mapPiecesAdded = new Queue<GameObject>();
    }

    public static MapManager Get() { return _this; }


    void SpawnRandomThing(float x, GameObject mapObj)
    {
        var vPos = new Vector3(x, Random.Range(32, 47), mapObj.transform.position.z);

        int item = Random.Range(0, 1+1);
        
        GameObject thing = null;
       
        if (item == 0 && x  > 90)
        {
                thing = Instantiate(_enemyPrefab, mapObj.transform);
             
        } else
        {
            thing = Instantiate(_gemPrefab, mapObj.transform);
        }

        thing.transform.position = vPos;
    }

    void SpawnThingsOnMap(GameObject mapObj)
    {
        //Place trees
        _currentTreeX = (_totalMapPiecesSpawned-1)*_blockSize + Random.Range(2, 3);

        while (_currentTreeX < (_totalMapPiecesSpawned*_blockSize) - 1)
        {

            bool bMakeTree = true;

            if (Random.Range(0,1+1) == 0)
            {
             //   bMakeTree = false;
            }

            if (_currentTreeX > 40 && _currentTreeX < 60)
            {
                //force it here
                bMakeTree = true;
            }
            
            GameObject treeObj = null;
            Vector3 vPos;

            bool bInitArea = _currentTreeX > 40 && _currentTreeX < 60;

            if (bMakeTree)
            {

                int type = Random.Range(0, 1 + 1);

                if (bInitArea)
                {
                    type = 1; //force a full size tree

                }
                if (type == 0)
                {

                    //tree from top of screen
                    treeObj = Instantiate(_rootFromholePrefab, mapObj.transform);
                    vPos = treeObj.transform.position;
                    vPos.x = _currentTreeX;
                    vPos.y = Random.Range(35, 45);

                    treeObj.transform.position = vPos;

                }
                else
                {
                    //tree from top of screen
                    treeObj = Instantiate(_treeWithRootPrefab, mapObj.transform);
                    vPos = treeObj.transform.position;
                    vPos.x = _currentTreeX;
                    treeObj.transform.position = vPos;
                }

                //oh, probably a good idea to actually grow the root
                RootManager rootManScript = treeObj.GetComponent<RootManager>();
                int segs = 30;

                int r = Random.Range(0, 2 + 1);
               
                
                if (bInitArea)
                {
                    //force it here
                    r = 2;
                }

                if (r == 1) segs = 45;
                if (r == 2) segs = 60;

                if (type == 0 && r == 2)
                {
                    //this is a hole starting from mid screen, don't make it this long
                    segs = 20;
                }
                if (_startingPointOnRoot == Vector2.zero)
                {
                  //  r = 2;
                }

                if (r == 2 && _currentTreeX < 100 && _currentTreeX > 50)
                {
                    _startingPointOnRoot = new Vector2(vPos.x, vPos.y);
                }

                rootManScript.GrowRoot(segs);

            }


            //between the trees is a good place to spawn "things"

            int thingsToSpawn = Random.Range(0, 1 + ((int)_currentTreeX / 50));

            for (int i = 0; i < thingsToSpawn; i++)
            {

                SpawnRandomThing(_currentTreeX + Random.Range(-6, 6.0f), mapObj);
            }
            //Debug.Log("Spawning tree at " + vPos);

            _currentTreeX += Random.Range(_distanceFromLastTreeMin, _distanceFromLastTreeMax);

        }

    }

    public void ResetGame()
    {
     
       while (_mapPiecesAdded.Count > 0) 
        {
            Destroy(_mapPiecesAdded.Dequeue());
        }

        _totalMapPiecesSpawned = 0;
        _currentTreeX = 0;
        _spawnX = 0;
        _startingPointOnRoot = Vector2.zero;

        CheckIfWeNeedToSpawnAMapPiece();
      
    }
    public Vector2 GetPlaceOnRoot() { return _startingPointOnRoot; }

    void CheckIfWeNeedToSpawnAMapPiece()
    {
        if (_player.transform.position.x + (renderAheadDistance/2) > _spawnX)
        {
           // Debug.Log("Spawn piece");

            var mapPiece = Instantiate(_mapPieceSmall);
            _totalMapPiecesSpawned++;

             var vPos = mapPiece.transform.position;
            vPos.x = _spawnX+ (_blockSize/2); //map pieces are centered in the prefab
            mapPiece.transform.position = vPos;
            _spawnX += _blockSize;
            _mapPiecesAdded.Enqueue(mapPiece);

            //Oh, it would be pretty boring if nothing was on the map, so let's spawn stuff on it too

            SpawnThingsOnMap(mapPiece);

            if (_mapPiecesAdded.Count > _maxPiecesToExistAtOnce)
            {
                Destroy(_mapPiecesAdded.Dequeue());
            }
        }
    }

    // Update is called once per frame
    void Update()
    {
        CheckIfWeNeedToSpawnAMapPiece();
    }
}
