using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.PlayerLoop;

public class Player : MonoBehaviour
{
    bool _isOnRoot = false;
    Rigidbody2D _rigid2D;
    FixedJoint2D _joint2D;
    float _rootIgnoreTimer;
    CapsuleCollider2D _collider;
    float _floatPowerHorizontal = 2000;
    float _maxHorizontalSpeed = 20;

    float _floatPowerVertical = 0;
    public SpriteRenderer _spriteRenderer;
    bool _movingRight;
    GameObject _rootWeAreOn = null;
    bool _playerIsDead = false;
    public List<Sprite> _sprites = new List<Sprite>();
    static Player _this;
    public SpriteRenderer _deathOverlay;
    public TMPro.TextMeshProUGUI _gemText;
    public TMPro.TextMeshProUGUI _metersText;
    public TMPro.TextMeshProUGUI _highText;
    Vector2 _vPlayerStartPos = new Vector2(50, 50);
    int _highMeters = 0;
    float _meters;
    float _lastPosX;
    int _doubleJumpsDone = 0;
    int _maxDoubleJumps = 1;
    Vector2 _jumpVelocity = new Vector2(30, 13);
    int _gemCount = 0;        

    static public Player Get() { return _this; }
    enum eSprites
    {
        HANG_LEFT = 0,
        HANG_RIGHT,
        FLY
    };

    private void Awake()
    {
        _this= this;
        _rigid2D = GetComponent<Rigidbody2D>();
        _collider= GetComponent<CapsuleCollider2D>();
    }

    void ResetPlayer()
    {
        _doubleJumpsDone = 0;
        _meters = 0;
        DisconnectFromRoot();
        _rootIgnoreTimer = 0;
        transform.localPosition = new Vector3(50, 43, 0);

        _lastPosX = transform.localPosition.x;
    }

    // Update is called once per frame
    
    void ConnectToRoot(GameObject target)
    {
        _isOnRoot = true;
        _doubleJumpsDone = 0;
        //move closer
      
        /*
        var vPos = transform.position;
        vPos.x = target.transform.position.x-0.3f;
        vPos.y = target.transform.position.y;

        transform.position = vPos;
        */

        _joint2D = gameObject.AddComponent<FixedJoint2D>();
        _joint2D.connectedBody = target.GetComponent<Rigidbody2D>();
        _rootWeAreOn = target.transform.parent.gameObject;
        
    }

    void DisconnectFromRoot()
    {
        if (_rootWeAreOn != null)
        {
            _rootWeAreOn.GetComponent<RootManager>().DisableRootCollision();
            RTMessageManager.Get().Schedule(0.6f, _rootWeAreOn.GetComponent<RootManager>().EnableRootCollision);

            _rootWeAreOn = null;
        }
        _isOnRoot = false;
        Destroy(_joint2D);
    }

  /*  void IgnoreRootsForABit(float timeToIgnore)
    {
        _rootIgnoreTimer = Time.time + timeToIgnore;
        _collider.isTrigger = true;
    }
*/
    private void OnCollisionEnter2D(Collision2D collision)
    {
       // Debug.Log("Colliding with " + collision.gameObject.name);

        if (collision.gameObject.CompareTag("Enemy"))
        {
            KillPlayer();
            return;
        }

        if (collision.gameObject.CompareTag("Obstacle"))
        {
          
            return;
        }

        if (!_isOnRoot)
        {
            if (!Input.GetKey(KeyCode.PageDown))
            {
                ConnectToRoot(collision.gameObject);
            }
        }

    }

    public void ModGemsAndUpdateText(int mod)
    {
        _gemCount += mod;
        _gemText.text = _gemCount.ToString();
    }

    private void OnTriggerEnter2D(Collider2D collision)
    {

      //  Debug.Log("Triggered by " + collision.gameObject.name + " which is a " + collision.gameObject.tag);

        if (collision.gameObject.CompareTag("Gem"))
        { 
            Gem gemScript = collision.gameObject.GetComponent<Gem>();
            gemScript.OnCollectGem();
            return;
        }

        if (collision.gameObject.CompareTag("Enemy"))
        {
            KillPlayer();
            return;
        }

        if (!_isOnRoot )
        {
            if (!Input.GetKey(KeyCode.PageDown))
            {
               // ConnectToRoot(collision.gameObject);
            } else
            {
               // Debug.Log("Skipping root");
            }
        }
    }

    void SetCanHangOnRoots(bool bNew)
    {
        if (bNew == false && _playerIsDead) return;
        _collider.isTrigger = !bNew;
    }

    public void ResetGame()
    {
        if (_isOnRoot)
            DisconnectFromRoot();
        _deathOverlay.enabled = false;


        transform.position = _vPlayerStartPos;
        SetCanHangOnRoots(true);
        _rigid2D.velocity = Vector2.zero;
        _rigid2D.angularVelocity = 0f;
        MapManager.Get().ResetGame();
    
        RTMessageManager.Get().Schedule(0.05f, PutPlayerInSafePlace);
        RTMessageManager.Get().Schedule(0.08f, Gopher.Get().ResetGame);
    }

    void ProcessInput()
    {
        if (Input.GetKeyDown(KeyCode.R))
        {
            ResetGame();

        }

        if (_playerIsDead) return;

        if (Input.GetKeyDown(KeyCode.LeftControl) || Input.GetKeyDown(KeyCode.RightControl))
        {
            if (_isOnRoot)
            {
                //jump off
                DisconnectFromRoot();

                _rigid2D.AddForce(new Vector2(7, 7), ForceMode2D.Impulse);
                //_rigid2D.velocity = _jumpVelocity;
                // IgnoreRootsForABit(0.3f);
            }
            else
            {
                //Double jump?  They clicked jump but they are in the air
                if (_doubleJumpsDone < _maxDoubleJumps)
                {
                    _rigid2D.AddForce(new Vector2(4, 7), ForceMode2D.Impulse);
                    //_rigid2D.velocity = _jumpVelocity;
                    _doubleJumpsDone++;
                }
            }

        }


        /*
        if (Input.GetKey(KeyCode.DownArrow))
        {
            //let go of rope they are on
            if (_isOnRoot)
            {
                DisconnectFromRoot();
            }

        }
        */

        /*
         if (Input.GetKeyDown(KeyCode.DownArrow))
         {
             SetCanHangOnRoots(false);
         }

         if (Input.GetKeyUp(KeyCode.DownArrow))
         {
             SetCanHangOnRoots(true);
         }
        */

        bool _usedArrowKey = false;

        if (!_isOnRoot)
        {

            //air float controls

            if (Input.GetKey(KeyCode.LeftArrow) || Input.GetKey(KeyCode.A))
            {
                _rigid2D.AddForce(new Vector2(-1 * _floatPowerHorizontal * Time.deltaTime, 0), ForceMode2D.Force);
                _usedArrowKey = true;
            }
            if (Input.GetKey(KeyCode.RightArrow) || Input.GetKey(KeyCode.D))
            {
                _rigid2D.AddForce(new Vector2(1 * _floatPowerHorizontal * Time.deltaTime, 0), ForceMode2D.Force);
                _usedArrowKey = true;
            }

            if (Input.GetKey(KeyCode.UpArrow) || Input.GetKey(KeyCode.W))
            {
                _rigid2D.AddForce(new Vector2(0, 1 * _floatPowerVertical*Time.deltaTime), ForceMode2D.Force);
                _usedArrowKey = true;
            }

            if (Input.GetKey(KeyCode.DownArrow) || Input.GetKey(KeyCode.S))
            {
                _rigid2D.AddForce(new Vector2(0, -1 * _floatPowerVertical * Time.deltaTime), ForceMode2D.Force);
                _usedArrowKey = true;
            }


        }
        else
        {
            //using arrow keys but ON a root


            if (Input.GetKeyDown(KeyCode.LeftArrow) || Input.GetKeyDown(KeyCode.A))
            {
                _usedArrowKey = true;
            }
            if (Input.GetKeyDown(KeyCode.RightArrow) || Input.GetKeyDown(KeyCode.D))
            {
                _usedArrowKey = true;
            }

            /*
            if (Input.GetKey(KeyCode.UpArrow) || Input.GetKey(KeyCode.W))
            {
                _usedArrowKey = true;
            }

            if (Input.GetKey(KeyCode.DownArrow) || Input.GetKey(KeyCode.S))
            {
                _usedArrowKey = true;
            }
            */

            if (_usedArrowKey && _isOnRoot)
            {
                DisconnectFromRoot();
            }

           

        }

        if (_rigid2D.velocity.x > _maxHorizontalSpeed)
        {

            var vVel = _rigid2D.velocity;

            vVel.x = _maxHorizontalSpeed;
            _rigid2D.velocity = vVel;
        }

    }

    public void KillPlayer()
    {
        if (_playerIsDead) return;

        RTAudioManager.Get().PlayEx("die", 1);
        RTAudioManager.Get().StopMusic();
        _playerIsDead = true;
        _deathOverlay.enabled = true;
        //disable physics
       // _collider.enabled = false;
        RTMessageManager.Get().Schedule(2, ResetGame);
    }
    void UpdateSpriteVisuals()
    {

        float C_MINIMUM_SPEED_NEEDED_TO_REGISTER_FACING = 2;
        if (Mathf.Abs(_rigid2D.velocity.x) > C_MINIMUM_SPEED_NEEDED_TO_REGISTER_FACING)
        _movingRight = _rigid2D.velocity.x > 0;

        //Debug.Log("Speed of player is " + _rigid2D.velocity);
        if (_isOnRoot)
        {
            if (_movingRight)
            {
                _spriteRenderer.sprite = _sprites[(int)eSprites.HANG_RIGHT];
            }
            else
            {
                _spriteRenderer.sprite = _sprites[(int)eSprites.HANG_LEFT];
            }
        } else
        {
            _spriteRenderer.sprite = _sprites[(int)eSprites.FLY];
         
        }

    }

    void PutPlayerInSafePlace()
    {
        ResetPlayer();
        //Debug.Log("moving player to known safe place, hopefully");

        var vPos = transform.position;
        Vector2 vPosOnTree = MapManager.Get().GetPlaceOnRoot();
        //Debug.Log("Place on tree is " + vPosOnTree);
        if (vPosOnTree == Vector2.zero)
        {
            vPosOnTree = _vPlayerStartPos;
        }

        vPos = new Vector3(vPosOnTree.x - 7.5f, vPosOnTree.y-10, vPos.z);
        //hope to the right
        transform.position= vPos;
        //_rigid2D.AddForce(new Vector2(15, 1), ForceMode2D.Impulse);
        _rigid2D.velocity = _jumpVelocity;
        _playerIsDead = false;
        _deathOverlay.enabled = false;
        _collider.enabled = true;

        RTAudioManager.Get().PlayMusic("gameplay", 0.7f, 1.0f, true);
    }

    void Start()
    {
        ResetGame();
       // RTMessageManager.Get().Schedule(0.5f, PutPlayerInSafePlace);
    }

    public int GetMetersRun()
    {
        return ((int)_meters);
    }

    void Update()
    {

        if (_rootIgnoreTimer != 0)
        {
            //we're currently ignoring roots
            if (_rootIgnoreTimer < Time.time) 
            {
                _collider.isTrigger = false;
                _rootIgnoreTimer = 0;
            }

        }

        ProcessInput();

        UpdateSpriteVisuals();

        if (!_playerIsDead)
        {
            _meters += transform.position.x - _lastPosX;

            _lastPosX = transform.position.x;
         
            _metersText.text = GetMetersRun().ToString() + " M";

            if (GetMetersRun() > _highMeters)
            {
                _highMeters = GetMetersRun();
                _highText.text = "HIGH: " + GetMetersRun().ToString() + " M";
            }
        }
    }




}

