using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RootManager : MonoBehaviour
{
    // Start is called before the first frame update
    public float segmentLength = 0.5f; // length of each segment
    public float angularVelocity = 100f; // angular velocity of the swinging motion
    public Sprite segmentSprite; // sprite for each segment

    private Rigidbody2D[] segments; // array of Rigidbody2D components of each segment

    private void Start()
    {
    }

    public void GrowRoot(int segmentCount)
    {
       //transform.localRotation = Quaternion.Euler(0, 0, Random.Range(-100, 100));

        segments = new Rigidbody2D[segmentCount];
        for (int i = 0; i < segmentCount; i++)
        {
            GameObject segmentObject = new GameObject("Segment " + i);
            segmentObject.transform.parent = transform;
            segmentObject.transform.position = transform.position - new Vector3(i * segmentLength, i * segmentLength, 0);
            Rigidbody2D segmentRigidbody = segmentObject.AddComponent<Rigidbody2D>();
            segmentRigidbody.interpolation = RigidbodyInterpolation2D.Interpolate;

            CircleCollider2D circleCollider = segmentObject.AddComponent<CircleCollider2D>();
            circleCollider.radius = 0.15f;
            segmentRigidbody.mass = 3;

            if (i > 0)
            {
                //connect to parent above us
                HingeJoint2D hingeJoint = segmentObject.AddComponent<HingeJoint2D>();
                hingeJoint.connectedBody = segments[i - 1];
                JointAngleLimits2D limits = new JointAngleLimits2D();
                limits.min = -45f;
                limits.max = 45f;
                hingeJoint.limits = limits;
                segmentRigidbody.bodyType = RigidbodyType2D.Dynamic;
               // hingeJoint.breakForce = 2000;
                hingeJoint.breakTorque = 2000;
            }
            else
            {
                //we don't move
                segmentRigidbody.bodyType = RigidbodyType2D.Static;
            }

            segments[i] = segmentRigidbody;
            GameObject visualObj = new GameObject("Visual");

            visualObj.transform.parent = segmentObject.transform;
            visualObj.transform.localPosition = new Vector3(0, 0, 0);

            SpriteRenderer spriteRenderer = visualObj.AddComponent<SpriteRenderer>();
            spriteRenderer.sprite = segmentSprite;

            visualObj.transform.localRotation = Quaternion.Euler(0, 0, -50);
            //visualObj.transform.localRotation = Quaternion.Euler(0, 0, Random.Range(-100, 100));
        }
    }
    public void DisableRootCollision()
    {
        foreach (var segment in segments) 
        { 
            segment.GetComponent<CircleCollider2D>().enabled= false;
        }
    }


    public void EnableRootCollision()
    {
        foreach (var segment in segments)
        {
            segment.GetComponent<CircleCollider2D>().enabled =true;
        }
    }

    private void Update()
    {
        // segments[segmentCount-1].AddForce(new Vector2(angularVelocity,0));
    }
}
